#include <stdio.h>
#include <stdlib.h>

#include <gnome.h>

#include "wp_prefs.h"
#include "wp.h"
#include "wp_view.h"


WPPreferences *preferences = NULL;


void gnome_config_set_color (char *path, GdkColor *c)
{
  float red_f = (float) c->red / (float) 0xFFFF;
  float green_f = (float) c->green / (float) 0xFFFF;
  float blue_f = (float) c->blue / (float) 0xFFFF;

  char *red_path = g_strdup_printf ("%s.red", path);
  char *green_path = g_strdup_printf ("%s.green", path);
  char *blue_path = g_strdup_printf ("%s.blue", path);

  gnome_config_set_float (red_path, red_f);
  gnome_config_set_float (green_path, green_f);
  gnome_config_set_float (blue_path, blue_f);

  free (red_path);
  free (green_path);
  free (blue_path);
}


GdkColor *gnome_config_get_color (char *path,
				  guint32 default_red,
				  guint32 default_green,
				  guint32 default_blue,
				  GdkColormap *colormap)
{
  float default_red_f = (float) default_red / (float) 0xFFFF;
  float default_green_f = (float) default_green / (float) 0xFFFF;
  float default_blue_f = (float) default_blue / (float) 0xFFFF;

  char *red_path = g_strdup_printf ("%s.red=%f", path, default_red_f);
  char *green_path = g_strdup_printf ("%s.green=%f", path, default_green_f);
  char *blue_path = g_strdup_printf ("%s.blue=%f", path, default_blue_f);

  float red_f = gnome_config_get_float (red_path);
  float green_f = gnome_config_get_float (green_path);
  float blue_f = gnome_config_get_float (blue_path);

  GdkColor *c = (GdkColor *) malloc (sizeof (GdkColor));
  c->red = (gint) (red_f * 0xFFFF);
  c->green = (gint) (green_f * 0xFFFF);
  c->blue = (gint) (blue_f * 0xFFFF);
  gdk_color_alloc (colormap, c);

  free (red_path);
  free (green_path);
  free (blue_path);

  return c;
}


static void copy_color (GdkColor *to, GdkColor *from, GdkColormap *colormap)
{
  to->red = from->red;
  to->green = from->green;
  to->blue = from->blue;
  gdk_color_alloc (colormap, to);
}


WPPreferences::WPPreferences ()
{
  draw_fast = gnome_config_get_bool ("/gwp/Preferences/DrawFast=false");
  mdi_mode = NULL; /* XXX */
  show_rulers = gnome_config_get_bool ("/gwp/Preferences/Rulers=true");

  GdkColormap *colormap = gtk_widget_get_default_colormap ();

  margin_color =
    gnome_config_get_color ("/gwp/Preferences/MarginColor",
			    0x9000, 0x9000, 0x9000, colormap);
  background_color =
    gnome_config_get_color ("/gwp/Preferences/BackGroundColor",
			    0xA5A5, 0xA5A5, 0xA5A5, colormap);
  foreground_color =
    gnome_config_get_color ("/gwp/Preferences/ForeGroundColor",
			    0x0000, 0x0000, 0x0000, colormap);
  cursor_color =
    gnome_config_get_color ("/gwp/Preferences/CursorColor",
			    0x0000, 0x8000, 0x4000, colormap);

  properties_dialog = NULL;
  notebook = NULL;

  b_margin_color = (GdkColor *) malloc (sizeof (GdkColor));
  b_background_color = (GdkColor *) malloc (sizeof (GdkColor));
  b_foreground_color = (GdkColor *) malloc (sizeof (GdkColor));
  b_cursor_color = (GdkColor *) malloc (sizeof (GdkColor));
}


WPPreferences::~WPPreferences ()
{
}


void WPPreferences::update_views ()
{
  for (GList *mdi_child = mdi->children;
       mdi_child;
       mdi_child = mdi_child->next)
    {
      GnomeMDIChild *mdic = GNOME_MDI_CHILD (mdi_child->data);

      for (GList *viewp = mdic->views; viewp; viewp = viewp->next)
	{
	  WPView *view = WP_VIEW (viewp->data);
	  wp_view_preferences_changed (view);
	}
    }
}


void WPPreferences::backup ()
{
  GdkColormap *colormap = gtk_widget_get_default_colormap ();

  b_draw_fast = draw_fast;
  copy_color (b_margin_color, margin_color, colormap);
  copy_color (b_background_color, background_color, colormap);
  copy_color (b_foreground_color, foreground_color, colormap);
  copy_color (b_cursor_color, cursor_color, colormap);
  b_mdi_mode = mdi_mode;
  b_show_rulers = show_rulers;
}


void WPPreferences::apply ()
{
  GdkColormap *colormap = gtk_widget_get_default_colormap ();

  draw_fast = b_draw_fast;
  copy_color (margin_color, b_margin_color, colormap);
  copy_color (background_color, b_background_color, colormap);
  copy_color (foreground_color, b_foreground_color, colormap);
  copy_color (cursor_color, b_cursor_color, colormap);
  mdi_mode = b_mdi_mode;
  show_rulers = b_show_rulers;

  gnome_config_set_bool ("/gwp/Preferences/DrawFast", draw_fast);
  gnome_config_set_bool ("/gwp/Preferences/Rulers", show_rulers);
  gnome_config_set_color ("/gwp/Preferences/MarginColor", margin_color);
  gnome_config_set_color ("/gwp/Preferences/BackGroundColor",
			  background_color);
  gnome_config_set_color ("/gwp/Preferences/ForeGroundColor",
			  foreground_color);
  gnome_config_set_color ("/gwp/Preferences/CursorColor", cursor_color);
  gnome_config_sync ();
  update_views ();
}


void 
preferences_ok_callback (GtkWidget */*widget*/, gpointer client_data)
{
  GtkWidget *properties_dialog = (GtkWidget *) client_data;
  gtk_widget_hide (properties_dialog);
  preferences->apply ();
}


void 
preferences_apply_callback (GtkWidget */*widget*/, gpointer /*client_data*/)
{
  preferences->apply ();
}


void 
preferences_cancel_callback (GtkWidget */*widget*/, gpointer client_data)
{
  GtkWidget *properties_dialog = (GtkWidget *) client_data;
  gtk_widget_hide (properties_dialog);
}


static void
show_rulers_toggle_callback (GtkWidget *widget, gpointer /*client_data*/)
{
  GtkCheckButton *cb = GTK_CHECK_BUTTON (widget);
  preferences->b_show_rulers = cb->toggle_button.active;
}

static GtkWidget *make_show_rulers_toggle ()
{
  GtkWidget *show_rulers_toggle =
    gtk_check_button_new_with_label ("Show Rulers");
  gtk_widget_show (show_rulers_toggle);

  gtk_signal_connect (GTK_OBJECT (show_rulers_toggle), "toggled",
		      (GtkSignalFunc) show_rulers_toggle_callback,
		      /*properties_dialog*/ NULL);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (show_rulers_toggle),
			       preferences->b_show_rulers);
  return show_rulers_toggle;
}


static void
draw_fast_toggle_callback (GtkWidget *widget, gpointer /*client_data*/)
{
  GtkCheckButton *cb = GTK_CHECK_BUTTON (widget);
  preferences->b_draw_fast = cb->toggle_button.active;
}

static GtkWidget *make_draw_fast_toggle ()
{
  GtkWidget *draw_fast_toggle =
    gtk_check_button_new_with_label ("Draw Fast");
  gtk_widget_show (draw_fast_toggle);

  gtk_signal_connect (GTK_OBJECT (draw_fast_toggle), "toggled",
		      (GtkSignalFunc) draw_fast_toggle_callback,
		      /*properties_dialog*/ NULL);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (draw_fast_toggle),
			       preferences->b_draw_fast);
  return draw_fast_toggle;
}


static void
color_selector_cb (GnomeColorPicker *cp,
		   gint /*r*/,
		   gint /*g*/,
		   gint /*b*/,
		   gint /*a*/,
		   GdkColor **color)
{
  gushort red, green, blue, alpha;
  gnome_color_picker_get_i16 (cp, &red, &green, &blue, &alpha);
  GdkColormap *colormap = gtk_widget_get_default_colormap ();

  (*color)->red = red;
  (*color)->green = green;
  (*color)->blue = blue;
  gdk_color_alloc (colormap, (*color));
}


static GtkWidget *make_color_selector (char *title, GdkColor **color)
{
  GtkWidget *hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox);

  GtkWidget *cp = gnome_color_picker_new ();
  gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (cp),
			      (*color)->red,
			      (*color)->green,
			      (*color)->blue, 0xFFFF);
  gtk_widget_show (cp);
  gtk_signal_connect (GTK_OBJECT (cp), "color_set",
		      GTK_SIGNAL_FUNC (color_selector_cb), color);

  GtkWidget *label = gtk_label_new (title);
  gtk_widget_show (label);

  gtk_box_pack_start (GTK_BOX (hbox), cp, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);

  return hbox;
}


void WPPreferences::showDialog ()
{
  GtkWidget *button;

  if (properties_dialog)
    {
      gtk_widget_show (properties_dialog);
      return;
    }

  properties_dialog = gtk_dialog_new();
  notebook = gtk_notebook_new();

  preferences->backup ();

  button = gtk_button_new_with_label ("OK");
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      (GtkSignalFunc) preferences_ok_callback,
                      properties_dialog);

  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (properties_dialog)->action_area),
		      button, TRUE, TRUE, 0);
  gtk_widget_grab_default (button);
  gtk_widget_show (button);

  button = gtk_button_new_with_label ("Apply");
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      (GtkSignalFunc) preferences_apply_callback,
                      properties_dialog);

  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (properties_dialog)->action_area),
		      button, TRUE, TRUE, 0);
  gtk_widget_show (button);

  button = gtk_button_new_with_label ("Cancel");
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      (GtkSignalFunc) preferences_cancel_callback,
                      properties_dialog);

  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (properties_dialog)->action_area),
		      button, TRUE, TRUE, 0);
  gtk_widget_show (button);

  {
    GtkWidget *label;
    GtkWidget *frame;
	  
    frame = gtk_frame_new ("Display");
    gtk_container_border_width (GTK_CONTAINER (frame), 10);
    /*gtk_widget_set_usize (frame, 200, 150);*/
    gtk_widget_show (frame);

    GtkWidget *vbox = gtk_vbox_new (FALSE, 0);
    gtk_widget_show (vbox);

    GtkWidget *draw_fast_toggle = make_draw_fast_toggle ();
    gtk_box_pack_start (GTK_BOX (vbox), draw_fast_toggle, FALSE, FALSE, 0);

    GtkWidget *show_rulers_toggle = make_show_rulers_toggle ();
    gtk_box_pack_start (GTK_BOX (vbox), show_rulers_toggle, FALSE, FALSE, 0);

    GtkWidget *margin_cp =
      make_color_selector ("Margin Color", &(preferences->b_margin_color));
    gtk_box_pack_start (GTK_BOX (vbox), margin_cp, FALSE, FALSE, 0);

    GtkWidget *background_cp =
      make_color_selector ("Background Color",
			   &(preferences->b_background_color));
    gtk_box_pack_start (GTK_BOX (vbox), background_cp, FALSE, FALSE, 0);

    GtkWidget *foreground_cp =
      make_color_selector ("Foreground Color",
			   &(preferences->b_foreground_color));
    gtk_box_pack_start (GTK_BOX (vbox), foreground_cp, FALSE, FALSE, 0);

    GtkWidget *cursor_cp =
      make_color_selector ("Cursor Color", &(preferences->b_cursor_color));
    gtk_box_pack_start (GTK_BOX (vbox), cursor_cp, FALSE, FALSE, 0);


    gtk_container_add (GTK_CONTAINER (frame), vbox);


    /*
    label = gtk_label_new ("Display0");
    gtk_container_add (GTK_CONTAINER (frame), label);
    gtk_widget_show (label);
    */
	  
    label = gtk_label_new ("Display");
    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), frame, label);
  }

  gtk_box_pack_start (GTK_BOX(GTK_DIALOG (properties_dialog)->vbox),
		      notebook, TRUE, TRUE, 0);
  gtk_widget_show(notebook);

  gtk_widget_show(properties_dialog);
}


void 
preferences_cmd_callback (GtkWidget */*widget*/, gpointer /*client_data*/)
{
  preferences->showDialog ();
}
